#!/bin/sh
#----------------------------------------------------------------------------
# /tmp/install.sh - installation script for Pure-FTPd
#
# Creation:     2002-11-06  jh
# Last Update:  $Id: install.sh 44740 2017-01-24 15:35:20Z ansgar $
#
# Copyright (c) 2002-2005 Juerg Hoh
# Copyright (c) 2006-2012 Ansgar Puester, ansgar.puester(at)freenet(dot)de
# Copyright (c) 2012-2015 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
# Copyright (c) 2016-2017 Ansgar Puester, ansgar.puester(at)freenet(dot)de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

# include eislib
. /var/install/include/eislib

# set package name
package_name=pure-ftpd

cert_path='/var/certs/ssl/certs'

# ---------------------------------------------------------------------------
# check pem file
# ---------------------------------------------------------------------------
check_pem_file()
{
   pem_file=$1

   is_file=''
   has_hd=''

   [ -f "$pem_file" ] && is_file='Y' || is_file='N'
   grep -q '^-----BEGIN DH PARAMETERS-----' $pem_file >/dev/null 2>&1
   [ $? = 0 ] && has_hd='Y' || has_hd='N'

   my_switch="$is_file$is_link$has_hd"

   case "$my_switch" in
      YY ) # file or link and DH PARAMETERS section
           ret=0
           ;;
      YN ) # file or link but no DH PARAMETERS section
           mecho --warn  "Warning check_pem_file: $pem_file has no DH PARAMETERS section."
           mecho --warn  "You can't use pure-ftpd when TLS is enabled."
           ret=1
           ;;
      *    )
           mecho --error "Error check_pem_file: Internal logic error (switch=$my_switch)"
           ret=2
           ;;

   esac
   return $ret
}

# ---------------------------------------------------------------------------
# generate symbolic links stop
# ---------------------------------------------------------------------------
cd /etc/rc2.d

ln -sf ../init.d/${package_name} K50${package_name}

# ---------------------------------------------------------------------------
# create ftp and vftp user
# ---------------------------------------------------------------------------

ftp_group='users';
ftp_group_id='100';

getent passwd ftp >/dev/null 2>&1
if [ "${?}" -ne "0" ]
then
    /var/install/bin/add-user ftp "*" 28 ${ftp_group_id} \
        "anonymous_ftp" /home/ftp /bin/false
fi

 # create home directory, that could have been removed by /tmp/preinstall
if [ ! -d /home/ftp ]
then
    mkdir /home/ftp
    chown ftp:users /home/ftp
fi

vftp_group='users';
vftp_group_id='100';

getent passwd vftp >/dev/null 2>&1
if [ "${?}" -ne "0" ]
then
    /var/install/bin/add-user vftp "*" 29 ${vftp_group_id} \
        "pureftp_virtual_users" /home/vftp /bin/false
fi


# ---------------------------------------------------------------------------
# update pure-ftpd.pem
# ---------------------------------------------------------------------------

#installed_pem=$cert_path/pure-ftpd.pem
#new_pem=/tmp/pure-ftpd.pem_2p3p0
#old_pems="/tmp/pure-ftpd.pem_1p5p2 /tmp/pure-ftpd.pem_1p5p4 /tmp/pure-ftpd.pem_1p7p0"

#if [ ! -f "$installed_pem" ]
#then
#    # pure-ftpd.pem does not exists, install latest version
#    mkdir -p cert_path
#    cp -p $new_pem $installed_pem
#else
#    # check installed file
#    for act_pem in $old_pems
#    do
#        if diff -q $installed_pem $act_pem >/dev/null
#        then
#            # installed file is an old version, install latest version
#            cp -p $new_pem $installed_pem
#        else
#            # installed file is self generated, do nothing
#            true
#        fi
#    done
#fi

# check_pem_file for HD PARAMETERS
#check_pem_file $installed_pem

# remove pem files from /tmp
#rm -f $new_pem
#rm -f $old_pems

# ---------------------------------------------------------------------------
# check pure-ftpd.pem
# ---------------------------------------------------------------------------
if [ -f $cert_path/pure-ftpd.pem ]
then
   check_pem_file $cert_path/pure-ftpd.pem
fi

# ---------------------------------------------------------------------------
# create objects for pure-ftpd
# ---------------------------------------------------------------------------
mkdir -p -m 0755 /etc/pure-ftpd

if [ ! -f /etc/pure-ftpd/pureftpd.pdb ]
then
   # install empty pureftpd.pdb
   mv /etc/pure-ftpd/pureftpd.pdb_empty /etc/pure-ftpd/pureftpd.pdb
else
   rm -f /etc/pure-ftpd/pureftpd.pdb_empty
fi

if [ ! -f /etc/ftpusers ]
then
   > /etc/ftpusers
fi


# ---------------------------------------------------------------------------
# menu
# ---------------------------------------------------------------------------
/var/install/bin/add-menu    \
        setup.services.menu  \
        setup.services.${package_name}.menu  \
        "Pure-FTPd Service"

# ---------------------------------------------------------------------------
# Create default config
# ---------------------------------------------------------------------------
if [ -f "/etc/config.d/${package_name}" ]
then
    update=true
fi

/var/install/config.d/${package_name}-update.sh

# ---------------------------------------------------------------------------
# start package with update
# ---------------------------------------------------------------------------
if ${update:-false}
then
    /var/install/config.d/${package_name}.sh --quiet
    /etc/init.d/${package_name} --quiet restart
fi

# remove old conf files
if [ -d /etc/pure-ftpd ]
then
    rm -f /etc/pureftpd.passwd
    rm -f /etc/pureftpd.pdb
fi

# ---------------------------------------------------------------------------
# end
# ---------------------------------------------------------------------------
