#!/bin/sh
#----------------------------------------------------------------------------
# /tmp/preinstall.sh - pre installation script for Pure-FTPd
#
# Creation:     2002-07-18  jh
# Last Update:  $Id: preinstall.sh 40318 2016-01-22 18:59:46Z hbfl $
#
# Copyright (c) 2002-2005 Juerg Hoh
# Copyright (c) 2006-2012 Ansgar Puester, ansgar.puester(at)freenet(dot)de
# Copyright (c) 2012-2015 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

# include eislib
. /var/install/include/eislib

package_name=pure_ftpd


# ---------------------------------------------------------------------------
# check for utf8 env
# ---------------------------------------------------------------------------
#codierung=$(grep  'LC_CTYPE' /etc/config.d/environment)
#if ! echo "$codierung" | grep -q 'utf8'
#then
#    mecho --warn "                                                                  "
#    mecho --warn " This version is for 'utf8' systems.                              "
#    mecho --warn "                                                                  "
#    anykey
#    exit 1
#fi


_check_version='/var/install/bin/check-version'

# Check base package version
package=base
base_minversion='1.8.5'
if [ ! -f ${_check_version} ] ||
   [ "$(${_check_version} ${package} ${base_minversion})" = "new" ]
then
    mecho -n --warn "The version of package ${package} "
    mecho --warn "must be ${minversion} or higher!"
    mecho --warn "Please update your eisfair system first!"
    anykey
    exit 1
fi

package=inet
minversion=1.16.0
if  [ "$(${_check_version} ${package} ${minversion})" = "new" ]
then
    mecho -n --info "The Pure-FTPd package was splitt from "
    mecho -n "'${package}' "
    mecho --info "package."
    mecho -n --warn "Please update the "
    mecho -n "'${package}' "
    mecho --warn "package first."
    anykey
    exit 1
fi



if [ -e /etc/init.d/proftpd ]
then
    mecho --info "Proftp detected"
    mecho --info "Will replace it by pure-ftpd"
    if [ -e /etc/init.d/proftpd ]
    then
       /etc/init.d/proftpd stop
    fi
    rm -f /etc/init.d/proftpd
    rm -f /etc/rc2.d/[SK]??proftpd
    rm -f /etc/proftpd.conf
    rm -f /usr/sbin/proftpd
fi

# remove old files

rm -f /var/install/prep/inet.pureftpd.checkvusers.sh
rm -f /var/install/prep/inet.pureftpd.listvusers.sh
rm -f /var/install/prep/inet.pureftpd.showvuser.sh
rm -f /var/install/prep/inet.pureftpd.pure-ftpwho.sh
rm -f /var/install/prep/inet.pureftpd.viewlog.sh
rm -f /var/install/servadm/inet.pureftpd.clean.sh
rm -f /var/install/servadm/inet.pureftpd.showvuser.sh
rm -f /var/install/servadm/inet.pureftpd.pure-ftpwho.sh
rm -f /var/install/servadm/inet.pureftpd.cert.sh
rm -f /var/install/form/inet.pureftpd.admin
rm -f /var/install/form/inet.pureftpd.checkvusers
rm -f /var/install/form/inet.pureftpd.listvusers
rm -f /var/install/form/inet.pureftpd.showvuser
rm -f /var/install/form/inet.pureftpd.pure-ftpwho
rm -f /var/install/form/inet.pureftpd.viewlog
rm -f /var/install/form/inet.pureftpd.cert
rm -f /var/install/bin/inet-list-vusers
rm -f /var/install/bin/inet-check-vusers
rm -f /var/install/bin/inet-show-vuser
rm -f /var/install/bin/inet-create-cert

# cp in new sysconfdir /etc/pure-ftpd/
if [ -f /etc/config.d/pure-ftpd ]
then
    if [ ! -d /etc/pure-ftpd ]
    then
        mkdir -p -m 0755 /etc/pure-ftpd
    fi

    if [ -f /etc/pureftpd.passwd ]
    then
        cp /etc/pureftpd.passwd /etc/pure-ftpd/pureftpd.passwd
    fi

    if [ -f /etc/pureftpd.pdb ]
    then
        cp /etc/pureftpd.pdb /etc/pure-ftpd/pureftpd.pdb
    fi
fi

if  [ "$(${_check_version} ${package_name} ${minversion})" = "not-installed" ]
then
    # remove some redundant files from inet version 1.0.6
    # unused files
    rm -f /home/ftp/welcome.msg
    # duplicate files
    rm -f /sbin/pure-ftpd
    rm -f /sbin/pure-ftpwho
    rm -f /sbin/pure-pw

    # files from old proftpd
    rm -f /usr/bin/ftpcount
    rm -f /usr/bin/ftpwho

    # remove man pages
    rm -f $RM_OPTION /usr/share/doc/inet/pure-ftpd.8
    rm -f $RM_OPTION /usr/share/doc/inet/pure-pw.8
    rm -f $RM_OPTION /usr/share/doc/inet/pure-ftpwho.8
    rm -f $RM_OPTION /usr/share/doc/inet/pure-quotacheck.8
    rm -f $RM_OPTION /usr/share/doc/inet/pure-statsdecode.8
    rm -f $RM_OPTION /usr/share/doc/inet/pure-uploadscript.8

    rm -f /var/install/servadm/change_pureftpd_status.sh

    if  [ -f  /etc/init.d/${package_name} ]
    then
        /etc/init.d/${package_name} stop
    fi
else

    if [ -f "/var/install/deinstall/${package_name}" ]
    then
        /var/install/bin/del-package -p ${package_name} --update
    fi
fi

rm -f /usr/sbin/pure-pw
rm -f /usr/sbin/pure-statsdecode

exit 0
# ---------------------------------------------------------------------------
# end
# ---------------------------------------------------------------------------
