#!/bin/sh
#----------------------------------------------------------------------------
# /var/install/config.d/pure-ftp-update.sh - pure-ftp parameter update script
#
# Creation:     2012-06-30 hbfl
# Last Update:  $Id: pure-ftpd-update.sh 44740 2017-01-24 15:35:20Z ansgar $
#
# Copyright (c) 2003-2012 Ansgar Puester, ansgar.puester(at)freenet(dot)de
# Copyright (c) 2012-2015 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
# Copyright (c) 2016-2016 Ansgar Puester, ansgar.puester(at)freenet(dot)de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

package_name="pure-ftpd"

# include configlib
. /var/install/include/configlib

cert_path='/var/certs/ssl/certs'

VERSION=$(grep "<version>" /var/install/packages/${package_name} |
          sed 's# ##g; s#</*version>##g')

# ---------------------------------------------------------------------------
# Set the default values for configuration
# ---------------------------------------------------------------------------
START_PURE_FTPD='no'
PURE_FTPD_START_METHOD='st'
PURE_FTPD_HARDKILL='no'
PURE_FTPD_PORT='21'
PURE_FTPD_NAT_MODE='no'
PURE_FTPD_WELCOMEFILE=''
PURE_FTPD_FSCHARSET='auto'

PURE_FTPD_UMASKS=''
PURE_FTPD_LIST_DOT_FILES='no'
PURE_FTPD_DISALLOW_RENAMING='no'
PURE_FTPD_DISALLOW_ANONYMOUS_UPLOAD='no'

PURE_FTPD_HANGUP_TIME='15'

PURE_FTPD_ALLOW_ROOT_ACCESS='no'
PURE_FTPD_ONLY_USERS='yes'
PURE_FTPD_ONLY_ANONYMOUS='no'
PURE_FTPD_DONT_CHROOT_GROUP=''
PURE_FTPD_TLS='0'
PURE_FTPD_CERTIFICATE='pure-ftpd.pem'
PURE_FTPD_NONANONYMOUS_IP=''

PURE_FTPD_MAX_USER='20'
PURE_FTPD_PARTITION_FILL_LIMIT='95'
PURE_FTPD_MAXCON_PER_IP=''
PURE_FTPD_MAX_CPU_LOAD=''
PURE_FTPD_MAX_LOGINS=''

PURE_FTPD_FILE_QUOTA=''
PURE_FTPD_USER_BANDWIDTH=''
PURE_FTPD_ANONYMOUS_BANDWIDTH=''
PURE_FTPD_ANONYMOUS_RATIO=''
PURE_FTPD_ALL_USER_RATIO=''

PURE_FTPD_LOG='no'
PURE_FTPD_LOG_FORMAT='CLF'
PURE_FTPD_LOG_PATH='/var/log/pure-ftpd.log'

PURE_FTPD_UPLOADSCRIPT_ARGS=''

PURE_FTPD_ENABLE_VIRTUAL_SERVER='no'
PURE_FTPD_VIRTUAL_SERVER_N='2'
PURE_FTPD_VIRTUAL_SERVER_1_NAME=''
PURE_FTPD_VIRTUAL_SERVER_1_ACTIVE='no'
PURE_FTPD_VIRTUAL_SERVER_1_IP='192.168.6.53'
PURE_FTPD_VIRTUAL_SERVER_1_DIR='/ftpdata/extern'
PURE_FTPD_VIRTUAL_SERVER_2_NAME=''
PURE_FTPD_VIRTUAL_SERVER_2_ACTIVE='no'
PURE_FTPD_VIRTUAL_SERVER_2_IP='192.168.1.53'
PURE_FTPD_VIRTUAL_SERVER_2_DIR='/ftpdata/intern'

PURE_FTPD_ADD_ARGS=''
PURE_FTPD_SHOW_ARGS='no'

PURE_FTPD_ENABLE_VIRTUAL_USERS='no'
PURE_FTPD_VIRTUAL_USERS_DELETE='no'
PURE_FTPD_VIRTUAL_USERS_N='2'
PURE_FTPD_VIRTUAL_USERS_1_USERNAME='testuser'
PURE_FTPD_VIRTUAL_USERS_1_ACTIVE='no'
PURE_FTPD_VIRTUAL_USERS_1_HOME='/home/vftp/testuser'
PURE_FTPD_VIRTUAL_USERS_1_PASSWD=''
PURE_FTPD_VIRTUAL_USERS_2_USERNAME='andereruser'
PURE_FTPD_VIRTUAL_USERS_2_ACTIVE='no'
PURE_FTPD_VIRTUAL_USERS_2_HOME='/home/vftp/andereruser'
PURE_FTPD_VIRTUAL_USERS_2_PASSWD=''


# ---------------------------------------------------------------------------
# read old variables
# ---------------------------------------------------------------------------
read_old_variables ()
{
    # read old values
    if [ -f /etc/config.d/${package_name} ]
    then
        /var/install/bin/backup-file --quiet ${package_name}
        . /etc/config.d/${package_name}
    else
        # new installation
        # PURE_FTPD_CERTIFICATE default is 'pure-ftpd.pem'
        if [ -f ${cert_path}/pure-ftpd.pem ]
        then
           # check if this is already a link
           check_for_link
        fi
    fi
}

# ---------------------------------------------------------------------------
# Write config and default files
# ---------------------------------------------------------------------------
make_config_file ()
{
    internal_conf_file="$1"
    {
    # -----------------------------------------------------------------------
    printgpl --conf "${package_name}" "2011-04-10" "ap" "Ansgar Puester"
    # -----------------------------------------------------------------------
    echo "# Pure-FTPd-VERSION ${VERSION}"
    # -----------------------------------------------------------------------
    printgroup "pure-ftpd (general settings)"
    # -----------------------------------------------------------------------


    # check if old config file given
    if [ -n "${START_FTP}" ]
    then
        old_ftp=true

        START_PURE_FTPD="${START_FTP}"
        PURE_FTPD_START_METHOD="${FTP_START_METHOD}"
        PURE_FTPD_HARDKILL="${FTP_HARDKILL}"
        PURE_FTPD_PORT="${FTP_PORT}"
        PURE_FTPD_NAT_MODE="${FTP_NAT_MODE}"
        PURE_FTPD_WELCOMEFILE="${FTP_WELCOMEFILE}"
    fi

    printvar "START_PURE_FTPD"
    printcomment             "start pure-ftpd 'yes' or 'no'"

    printvar "PURE_FTPD_START_METHOD"
    printcomment             "Start method for pure-ftpd."
    printcomment             "'st' start pure-ftpd as standalone server."
    printcomment             "'xi' start pure-ftpd via xinetd."
    printcomment             "'xi' requires START_XINETD='yes'."
    printvar "PURE_FTPD_HARDKILL"
    printcomment             "kill all pure-ftpd processes, when stopping"
    printcomment             "the main pure-ftpd daemon 'yes' or 'no'."
    printcomment             "  'yes' all pure-ftpd processes are killed"
    printcomment             "  'no'  only main pure-ftpd is killed"
    printcomment             "Requires PURE_FTPD_START_METHOD='st'."
    printvar "PURE_FTPD_PORT"
    printcomment             "Listen for an incoming connection on port"
    printcomment             "PURE_FTPD_PORT."
    printvar "PURE_FTPD_NAT_MODE"
    printcomment             "enable support for NAT 'yes' or 'no'"
    printvar "PURE_FTPD_WELCOMEFILE"
    printcomment             "Filename containing Welcome Message."
    printvar "PURE_FTPD_FSCHARSET"
    printcomment             "Filesystem character set."
    printcomment             "Values ISO-8859-15, UTF-8 or auto."

    # -----------------------------------------------------------------------
    printgroup "pure-ftpd (file related settings)"
    # -----------------------------------------------------------------------


    if ${old_ftp:-false}
    then
        PURE_FTPD_UMASKS="${FTP_UMASKS}"
        PURE_FTPD_LIST_DOT_FILES="${FTP_LIST_DOT_FILES}"
        PURE_FTPD_DISALLOW_RENAMING="${FTP_DISALLOW_RENAMING}"
        PURE_FTPD_DISALLOW_ANONYMOUS_UPLOAD="${FTP_DISALLOW_ANONYMOUS_UPLOAD}"

        PURE_FTPD_HANGUP_TIME="${FTP_HANGUP_TIME}"
    fi

    printvar "PURE_FTPD_UMASKS"
    printcomment             "Format <umask for files>:<umask for dirs>."
    printcomment             "Change the file creation mask."
    printcomment             "The default is 133:022."
    printvar "PURE_FTPD_LIST_DOT_FILES"
    printcomment             "List files beginning with a dot ('.')"
    printcomment             "even when the client doesn't append the"
    printcomment             "'-a' option to the list command."
    printcomment             "This is a workaround for badly configured"
    printcomment             "FTP clients."
    printvar "PURE_FTPD_DISALLOW_RENAMING"
    printcomment             "Disallow renaming of files."
    printvar "PURE_FTPD_DISALLOW_ANONYMOUS_UPLOAD"
    printcomment             "Disallow upload for anonymous users."

    # -----------------------------------------------------------------------
    printgroup "pure-ftpd (timeout settings)"
    # -----------------------------------------------------------------------

    printvar "PURE_FTPD_HANGUP_TIME"
    printcomment             "timeout in minutes"

    # -----------------------------------------------------------------------
    printgroup "pure-ftpd (access settings)"
    # -----------------------------------------------------------------------

    if ${old_ftp:-false}
    then
        PURE_FTPD_ALLOW_ROOT_ACCESS="${FTP_ALLOW_ROOT_ACCESS}"
        PURE_FTPD_ONLY_USERS="${FTP_ONLY_USERS}"
        PURE_FTPD_ONLY_ANONYMOUS="${FTP_ONLY_ANONYMOUS}"
        PURE_FTPD_DONT_CHROOT_GROUP="${FTP_DONT_CHROOT_GROUP}"
        PURE_FTPD_TLS="${FTP_TLS}"
        PURE_FTPD_NONANONYMOUS_IP="${FTP_NONANONYMOUS_IP}"
    fi

    printvar "PURE_FTPD_ALLOW_ROOT_ACCESS"
    printcomment             "can root do FTP 'yes' or 'no'"
    printvar "PURE_FTPD_ONLY_USERS"
    printcomment             "disallow anonymous FTP 'yes' or 'no'"
    printvar "PURE_FTPD_ONLY_ANONYMOUS"
    printcomment             "Only allow anonymous users."
    printvar "PURE_FTPD_DONT_CHROOT_GROUP"
    printcomment             "group, which is not chrooted (numerical!!)"
    printvar "PURE_FTPD_TLS"
    printcomment             "'0'  support for SSL/TLS is disabled."
    printcomment             "'1'  clients can connect either the"
    printcomment             "     traditional way or through an"
    printcomment             "     SSL/TLS layer."
    printcomment             "'2'  cleartext sessions are refused and only"
    printcomment             "     SSL/TLS compatible clients are accepted"
    printcomment             "'3'  cleartext sessions are refused and only"
    printcomment             "     SSL/TLS compatible clients are accepted."
    printcomment             "     Clear data connections are also refused,"
    printcomment             "     so private data connections are enforced."
    printvar "PURE_FTPD_CERTIFICATE"
    printcomment             "Name of cert file to be used"
    printcomment             "If name is not pure-ftpd.pem a link named pure-ftpd.pem"
    printcomment             "pointing to the file wil be created."
    printvar "PURE_FTPD_NONANONYMOUS_IP"
    printcomment             "Allow non-anonymous FTP access only on"
    printcomment             "this specific local IP address."
    printcomment             "Empty: No limitation."

    # -----------------------------------------------------------------------
    printgroup "pure-ftpd (limit settings)"
    # -----------------------------------------------------------------------


    if ${old_ftp:-false}
    then
        PURE_FTPD_MAX_USER="${FTP_MAX_USER}"
        PURE_FTPD_LIMIT="${FTP_LIMIT}"
        PURE_FTPD_MAXCON_PER_IP="${FTP_MAXCON_PER_IP}"
        PURE_FTPD_MAX_CPU_LOAD="${FTP_MAX_CPU_LOAD}"
        PURE_FTPD_MAX_LOGINS="${FTP_MAX_LOGINS}"
    fi

    # Version 2.2.0
    limit=${PURE_FTPD_LIMIT}

    if [ -n "${limit}" ]
    then
        PURE_FTPD_PARTITION_FILL_LIMIT=${PURE_FTPD_LIMIT}
    fi

    printvar "PURE_FTPD_MAX_USER"
    printcomment             "maximal logged-in users"
    printvar "PURE_FTPD_PARTITION_FILL_LIMIT"
    printcomment             "Don't allow uploads if the partition is more"
    printcomment             "than <percentage>% full."
    printcomment             "Using pure-ftpd's -k switch."
    printvar "PURE_FTPD_MAXCON_PER_IP"
    printcomment             "Maximum number of connections per IP."
    printcomment             "Limit the number of simultanous connections"
    printcomment             "coming from the same IP address to n."
    printcomment             "Requires PURE_FTPD_START_METHOD='st'."
    printvar "PURE_FTPD_MAX_CPU_LOAD"
    printcomment             "Don't allow anonymous download if the"
    printcomment             "load is above <cpu load> ."
    printcomment             "Upload is still allowed, though."
    printvar "PURE_FTPD_MAX_LOGINS"
    printcomment             "Format"
    printcomment             "<max user logins>:<max anonymous logins>."
    printcomment             "It restricts the number of concurrent"
    printcomment             "sessions the same user can have."
    printcomment             "A null value ('0') means 'unlimited'."


    # -----------------------------------------------------------------------
    printgroup "pure-ftpd (quota/bandwidth/ratio settings)"
    # -----------------------------------------------------------------------


    if ${old_ftp:-false}
    then
        PURE_FTPD_FILE_QUOTA="${FTP_FILE_QUOTA}"
        PURE_FTPD_USER_BANDWIDTH="${FTP_USER_BANDWIDTH}"
        PURE_FTPD_ANONYMOUS_BANDWIDTH="${FTP_ANONYMOUS_BANDWIDTH}"
        PURE_FTPD_ANONYMOUS_RATIO="${FTP_ANONYMOUS_RATIO}"
        PURE_FTPD_ALL_USER_RATIO="${FTP_ALL_USER_RATIO}"
    fi


    printvar "PURE_FTPD_FILE_QUOTA"
    printcomment             "PureFTPd's virtual quota mechanism."
    printcomment             "Format <max files>:<max size>."
    printcomment             "<max size> is in Megabytes."
    printcomment             "Quotas are enabled for all users, except"
    printcomment             "for users of trusted groups."
    printcomment             "See PURE_FTPD_DONT_CHROOT_GROUP."
    printcomment             "To create the required .ftpquota files"
    printcomment             "see pure-quotacheck."
    printvar "PURE_FTPD_USER_BANDWIDTH"
    printcomment             "Enable bandwidth limitation for all users"
    printcomment             "(Unix users, user anonymous and virtual users)."
    printcomment             "Format [<upload>]:[<download>]."
    printcomment             "Bandwidth is specified in kilobytes/seconds."
    printcomment             "Examples:"
    printcomment             "256:64 256KB/s for up-, 64KB/s for downloads"
    printcomment             "256:   256KB/s for up-, no limit for downloads"
    printcomment             ":64    no limit for up-, 64 KB/s for downloads"
    printvar "PURE_FTPD_ANONYMOUS_BANDWIDTH"
    printcomment             "Enable bandwidth limitation for user anonymous"
    printcomment             "See PURE_FTPD_USER_BANDWIDTH."
    printcomment             "Only one bandwidth limit (PURE_FTPD_USER_BANDWIDTH"
    printcomment             "or PURE_FTPD_ANONYMOUS_BANDWIDTH) can be set."
    printvar "PURE_FTPD_ANONYMOUS_RATIO"
    printcomment             "Enable ratios for user anonymous."
    printcomment             "Format <upload ratio>:<download ratio>."
    printcomment             "Ratio is specified in Mbyte."
    printcomment             "For example 2:5 means that an anonymousi"
    printcomment             "user has to upload at least 2 Mb of goodies"
    printcomment             "to be able to download 5 Mb."
    printvar "PURE_FTPD_ALL_USER_RATIO"
    printcomment             "Enable ratios for everybody (anonymous"
    printcomment             "and non-anonymous)."
    printcomment             "See PURE_FTPD_ANONYMOUS_RATIO."

    # -----------------------------------------------------------------------
    printgroup "pure-ftpd (log settings)"
    # -----------------------------------------------------------------------


    if ${old_ftp:-false}
    then
        PURE_FTPD_LOG="${FTP_LOG}"
        PURE_FTPD_LOG_FORMAT="${FTP_LOG_FORMAT}"
        PURE_FTPD_LOG_PATH="${FTP_LOG_PATH}"

        PURE_FTPD_UPLOADSCRIPT_ARGS="${FTP_UPLOADSCRIPT_ARGS}"
    fi


    printvar "PURE_FTPD_LOG"
    printcomment             "Enable('yes') or disable('no') recording of"
    printcomment             "all file transfers into a specific log file,"
    printcomment             "in an alternative format."
    printvar "PURE_FTPD_LOG_FORMAT"
    printcomment             "Format of alternative log file. The values"
    printcomment             "'CLF', 'Stats' 'W3C' and 'xferlog' are"
    printcomment             "allowed."
    printvar "PURE_FTPD_LOG_PATH"
    printcomment             "Log file name for alternative log file."

    # -----------------------------------------------------------------------
    printgroup "pure-ftpd (upload settings)"
    # -----------------------------------------------------------------------

    printvar "PURE_FTPD_UPLOADSCRIPT_ARGS"
    printcomment             "Arguments for pure-uploadscript."
    printcomment             "When set, pure-ftpd will be startet with"
    printcomment             "argument -o and pure-uploadscript"
    printcomment             "with argument \$PURE_FTPD_UPLOADSCRIPT_ARGS"
    printcomment             "will be startet in the background."
    printcomment             "Example '-r /tmp/scanner.sh'."
    printcomment             "Requires PURE_FTPD_START_METHOD='st'."

    # -----------------------------------------------------------------------
    printgroup "pure-ftpd (virtual server settings)"
    # -----------------------------------------------------------------------


    if [ -n "${FTP_ENABLE_VIRTUAL_SERVER}" ]
    then
        PURE_FTPD_ENABLE_VIRTUAL_SERVER="${FTP_ENABLE_VIRTUAL_SERVER}"
        PURE_FTPD_VIRTUAL_SERVER_N="${FTP_VIRTUAL_SERVER_N}"
    fi

    printvar "PURE_FTPD_ENABLE_VIRTUAL_SERVER"
    printcomment             "enable virtual server 'yes' or 'no'"

    printvar "PURE_FTPD_VIRTUAL_SERVER_N"
    printcomment             "number of virtual servers"

    idx="1"
    while [ ${idx} -le ${PURE_FTPD_VIRTUAL_SERVER_N} ]
    do
        if [ -n "${FTP_VIRTUAL_SERVER_N}" ]
        then
            eval PURE_FTPD_VIRTUAL_SERVER_${idx}_ACTIVE='yes'
            eval PURE_FTPD_VIRTUAL_SERVER_${idx}_IP='${FTP_VIRTUAL_SERVER_'${idx}'_IP}'
            eval PURE_FTPD_VIRTUAL_SERVER_${idx}_DIR='${FTP_VIRTUAL_SERVER_'${idx}'_DIR}'
        fi

        printvar "PURE_FTPD_VIRTUAL_SERVER_${idx}_NAME"
        printcomment             "Use a name, its only for you"
        printvar "PURE_FTPD_VIRTUAL_SERVER_${idx}_ACTIVE"
        printcomment             "Use: yes or no"
        printvar "PURE_FTPD_VIRTUAL_SERVER_${idx}_IP"
        printcomment             "IP address of the virtual server"
        printvar "PURE_FTPD_VIRTUAL_SERVER_${idx}_DIR"
        printcomment             "Directory for virtual server."
        idx=$(expr ${idx} + 1)
    done


    # -----------------------------------------------------------------------
    printgroup "pure-ftpd (additional settings)"
    # -----------------------------------------------------------------------

    if ${old_ftp:-false}
    then
        PURE_FTPD_ADD_ARGS="${FTP_ADD_ARGS}"
        PURE_FTPD_SHOW_ARGS="${FTP_SHOW_ARGS}"
    fi

    printvar "PURE_FTPD_ADD_ARGS"
    printcomment             "Additional arguments / switches for"
    printcomment             "pure-ftpd. See pure-ftpd documentation."
    printcomment             "Please use this option only if you know"
    printcomment             "what you are doing."
    printvar "PURE_FTPD_SHOW_ARGS"
    printcomment             "Show all arguments for pure-ftpd on startup."
    printcomment             "'yes' enables this debugging option."

    # -----------------------------------------------------------------------
    printcustomgroup "pure-ftpd (virtual Users)" << !EOC
    # This users aren't real system users, they only can do FTP
    #
    #   PURE_FTPD_VIRTUAL_USERS_x_PASSWD:
    #     If you set a password to '', you will be asked to
    #     enter the password when the user is created.
    #     If you define a password, the virtual user will
    #     be created without asking.
    #     To keep security the password will be changed
    #     to '******' after creating the user or
    #     changing the password.
    #     If you want to use the Webconf Package, you'll
    #     have to define passwords for all virtual users.
!EOC
    # -----------------------------------------------------------------------

    if ${old_ftp:-false}
    then
        PURE_FTPD_ENABLE_VIRTUAL_USERS="${FTP_ENABLE_VIRTUAL_USERS}"
        PURE_FTPD_VIRTUAL_USERS_DELETE="${FTP_VIRTUAL_USERS_DELETE}"
        PURE_FTPD_VIRTUAL_USERS_N="${FTP_VIRTUAL_USERS_N}"
    fi

    printvar "PURE_FTPD_ENABLE_VIRTUAL_USERS"
    printcomment             "enable virtual user 'yes' or 'no'"
    printvar "PURE_FTPD_VIRTUAL_USERS_DELETE"
    printcomment             "Delete virtual users that are no more listed"
    printcomment             "in a PURE_FTPD_VIRTUAL_USERS_#_USERNAME variable"
    printcomment             "If you set PURE_FTPD_VIRTUAL_USERS_DELETE to 'yes'"
    printcomment             "only those virtual users listed in the"
    printcomment             "actual configuration file will be available."
    printcomment             "Other virtual users will be deleted, but"
    printcomment             "their home directories will still be there."
    printcomment             "For compatiblity to older versions this"
    printcomment             "variable defaults to 'no'."
    printcomment             "This means that virtual users that are no"
    printcomment             "more listet in PURE_FTPD_VIRTUAL_USERS_#_USERNAME"
    printcomment             "variables are not deleted but this users are"
    printcomment             "still able to connect."


    printvar "PURE_FTPD_VIRTUAL_USERS_N"
    printcomment             "number of virtual users"

    idx="1"
    while [ ${idx} -le ${PURE_FTPD_VIRTUAL_USERS_N} ]
    do
        if ${old_ftp:-false}
        then
            eval PURE_FTPD_VIRTUAL_USERS_${idx}_USERNAME='${FTP_VIRTUAL_USERS_'${idx}'_USERNAME}'
            eval PURE_FTPD_VIRTUAL_USERS_${idx}_ACTIVE='yes'
            eval PURE_FTPD_VIRTUAL_USERS_${idx}_HOME='${FTP_VIRTUAL_USERS_'${idx}'_HOME}'
            eval PURE_FTPD_VIRTUAL_USERS_${idx}_PASSWD='${FTP_VIRTUAL_USERS_'${idx}'_PASSWD}'
        fi

        printvar "PURE_FTPD_VIRTUAL_USERS_${idx}_USERNAME"
        printcomment             "Username of the virtual user"
        printvar "PURE_FTPD_VIRTUAL_USERS_${idx}_ACTIVE"
        printcomment             "Use: yes or no"
        printvar "PURE_FTPD_VIRTUAL_USERS_${idx}_HOME"
        printcomment             "Home directory of the virtual user."
        printcomment             "Normally /home/vftp/<username>"
        printvar "PURE_FTPD_VIRTUAL_USERS_${idx}_PASSWD"
        printcomment             "Password of the virtual user"
        idx=$(expr ${idx} + 1)
    done

    echo
    # -----------------------------------------------------------------------
    printend
    # -----------------------------------------------------------------------
    } >  $internal_conf_file
    # Set rights
    chmod 0600 $internal_conf_file
    chown root $internal_conf_file
}


# ---------------------------------------------------------------------------
# Create the check.d file
# ---------------------------------------------------------------------------
make_check_file()
{
printgpl --check ${package_name} "2011-04-10" "ap" "Ansgar Puester" >/etc/check.d/${package_name}
cat >>/etc/check.d/${package_name} <<EOF
# Variable                           OPT_VARIABLE                       VARIABLE_N                     VALUE
START_PURE_FTPD                      -                                  -                              YESNO
PURE_FTPD_START_METHOD               START_PURE_FTPD                    -                              PURE_FTPD_STARTMODE
PURE_FTPD_HARDKILL                   START_PURE_FTPD                    -                              YESNO
PURE_FTPD_PORT                       START_PURE_FTPD                    -                              PORT
PURE_FTPD_NAT_MODE                   START_PURE_FTPD                    -                              YESNO
PURE_FTPD_WELCOMEFILE                START_PURE_FTPD                    -                              NONE
PURE_FTPD_FSCHARSET                  START_PURE_FTPD                    -                              PURE_FTPD_FSCHARSET
PURE_FTPD_UMASKS                     START_PURE_FTPD                    -                              PURE_FTPD_UMASKS
PURE_FTPD_LIST_DOT_FILES             START_PURE_FTPD                    -                              YESNO
PURE_FTPD_DISALLOW_RENAMING          START_PURE_FTPD                    -                              YESNO
PURE_FTPD_DISALLOW_ANONYMOUS_UPLOAD  START_PURE_FTPD                    -                              YESNO
PURE_FTPD_HANGUP_TIME                START_PURE_FTPD                    -                              NUMERIC
PURE_FTPD_ALLOW_ROOT_ACCESS          START_PURE_FTPD                    -                              YESNO
PURE_FTPD_ONLY_USERS                 START_PURE_FTPD                    -                              YESNO
PURE_FTPD_ONLY_ANONYMOUS             START_PURE_FTPD                    -                              YESNO
PURE_FTPD_DONT_CHROOT_GROUP          START_PURE_FTPD                    -                              PURE_FTPD_GID
PURE_FTPD_TLS                        START_PURE_FTPD                    -                              PURE_FTPD_TLS
PURE_FTPD_CERTIFICATE                START_PURE_FTPD                    -                              PURE_FTPD_CERTIFICATE
PURE_FTPD_NONANONYMOUS_IP            START_PURE_FTPD                    -                              EIPADDR
PURE_FTPD_MAX_USER                   START_PURE_FTPD                    -                              NUMERIC
PURE_FTPD_PARTITION_FILL_LIMIT       START_PURE_FTPD                    -                              PURE_FTPD_PERCENTAGE
PURE_FTPD_MAXCON_PER_IP              START_PURE_FTPD                    -                              ENUMERIC
PURE_FTPD_MAX_CPU_LOAD               START_PURE_FTPD                    -                              ENUMERIC
PURE_FTPD_MAX_LOGINS                 START_PURE_FTPD                    -                              PURE_FTPD_MAXLOGINS
PURE_FTPD_FILE_QUOTA                 START_PURE_FTPD                    -                              PURE_FTPD_FILEQUOTA
PURE_FTPD_USER_BANDWIDTH             START_PURE_FTPD                    -                              PURE_FTPD_BANDWIDTH
PURE_FTPD_ANONYMOUS_BANDWIDTH        START_PURE_FTPD                    -                              PURE_FTPD_BANDWIDTH
PURE_FTPD_ANONYMOUS_RATIO            START_PURE_FTPD                    -                              PURE_FTPD_RATIO
PURE_FTPD_ALL_USER_RATIO             START_PURE_FTPD                    -                              PURE_FTPD_RATIO
PURE_FTPD_LOG                        START_PURE_FTPD                    -                              YESNO
PURE_FTPD_LOG_FORMAT                 PURE_FTPD_LOG                      -                              PURE_FTPD_LOGFORMAT
PURE_FTPD_LOG_PATH                   PURE_FTPD_LOG                      -                              NOTEMPTY
PURE_FTPD_UPLOADSCRIPT_ARGS          START_PURE_FTPD                    -                              NONE
PURE_FTPD_ENABLE_VIRTUAL_SERVER      START_PURE_FTPD                    -                              YESNO
PURE_FTPD_VIRTUAL_SERVER_N           PURE_FTPD_ENABLE_VIRTUAL_SERVER    -                              NUMERIC
PURE_FTPD_VIRTUAL_SERVER_%_NAME      PURE_FTPD_ENABLE_VIRTUAL_SERVER    PURE_FTPD_VIRTUAL_SERVER_N     NONE
PURE_FTPD_VIRTUAL_SERVER_%_ACTIVE    PURE_FTPD_ENABLE_VIRTUAL_SERVER    PURE_FTPD_VIRTUAL_SERVER_N     YESNO
PURE_FTPD_VIRTUAL_SERVER_%_IP        PURE_FTPD_VIRTUAL_SERVER_%_ACTIVE  PURE_FTPD_VIRTUAL_SERVER_N     IPADDR
PURE_FTPD_VIRTUAL_SERVER_%_DIR       PURE_FTPD_VIRTUAL_SERVER_%_ACTIVE  PURE_FTPD_VIRTUAL_SERVER_N     NOBLANK
PURE_FTPD_ADD_ARGS                   START_PURE_FTPD                    -                              NONE
PURE_FTPD_SHOW_ARGS                  START_PURE_FTPD                    -                              YESNO
PURE_FTPD_ENABLE_VIRTUAL_USERS       START_PURE_FTPD                    -                              YESNO
PURE_FTPD_VIRTUAL_USERS_DELETE       PURE_FTPD_ENABLE_VIRTUAL_USERS     -                              YESNO
PURE_FTPD_VIRTUAL_USERS_N            PURE_FTPD_ENABLE_VIRTUAL_USERS     -                              NUMERIC
PURE_FTPD_VIRTUAL_USERS_%_USERNAME   PURE_FTPD_ENABLE_VIRTUAL_USERS     PURE_FTPD_VIRTUAL_USERS_N      NOBLANK
PURE_FTPD_VIRTUAL_USERS_%_ACTIVE     PURE_FTPD_ENABLE_VIRTUAL_USERS     PURE_FTPD_VIRTUAL_USERS_N      YESNO
PURE_FTPD_VIRTUAL_USERS_%_HOME       PURE_FTPD_VIRTUAL_USERS_%_ACTIVE   PURE_FTPD_VIRTUAL_USERS_N      NOBLANK
PURE_FTPD_VIRTUAL_USERS_%_PASSWD     PURE_FTPD_VIRTUAL_USERS_%_ACTIVE   PURE_FTPD_VIRTUAL_USERS_N      NONE
PURE_FTPD_VIRTUAL_USERS_%_PASSWD     PURE_FTPD_VIRTUAL_USERS_%_ACTIVE   PURE_FTPD_VIRTUAL_USERS_N      PASSWD
EOF

   # Set rights for check.d file
   chmod 0600 /etc/check.d/${package_name}
   chown root /etc/check.d/${package_name}



printgpl --check_exp ${package_name} "2011-04-10" "ap" "Ansgar Puester"  >/etc/check.d/${package_name}.exp
cat >>/etc/check.d/${package_name}.exp <<EOF
PURE_FTPD_STARTMODE     = 'st|xi'
                        : 'only st(standalone) or xi(xinetd) are allowed'
PURE_FTPD_FSCHARSET     = 'ISO-8859-15|UTF-8|auto'
                        : 'only ISO-8859-15, UTF-8 or auto is allowed'
PURE_FTPD_UMASK         = '[0-7]{3}'
                        : 'umask must be octal 000 to octal 777'
PURE_FTPD_UMASKS        = '()|(RE:PURE_FTPD_UMASK):(RE:PURE_FTPD_UMASK)'
                        : 'umasks format is <umask for files>:<umask for dirs> where umask is octal 000 to octal 777'
PURE_FTPD_GID           = '()|[0-9]{0,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]'
                        : 'group must be in range 1..65535'
PURE_FTPD_MAXLOGINS     = '()|(RE:NUMERIC):(RE:NUMERIC)'
                        : 'format is <max user logins>:<max anonymous logins>'
PURE_FTPD_FILEQUOTA     = '()|(RE:NUMERIC):(RE:NUMERIC)'
                        : 'format is <max files>:<max size>'
PURE_FTPD_BANDWIDTH     = '()|(RE:NUMERIC):()|():(RE:NUMERIC)|(RE:NUMERIC):(RE:NUMERIC)'
                        : 'format is [<upload>]:[<download>]'
PURE_FTPD_LOGFORMAT     = 'CLF|Stats|W3C|xferlog'
                        : 'only CLF, Stats, W3C and xferlog are allowed'
PURE_FTPD_RATIO         = '()|(RE:NUMERIC):(RE:NUMERIC)'
                        : ' format is <upload ratio>:<download ratio>'
PURE_FTPD_TLS           = '0|1|2|3'
                        : 'only value 0, 1, 2 or 3 is allowed'
PURE_FTPD_CERTIFICATE   = '(RE:NOTEMPTY)'
                        : 'SSL Server Certificat Name'
PURE_FTPD_PERCENTAGE    = '()|[1-9][0-9]|[1-9]'
                        : 'value must be in range 1..99'
EOF

    # Set rights for check.exp file
    chmod 0600 /etc/check.d/${package_name}.exp
    chown root /etc/check.d/${package_name}.exp



printgpl --check_ext ${package_name} "2011-04-10" "ap" "Ansgar Puester"   >/etc/check.d/${package_name}.ext
cat >>/etc/check.d/${package_name}.ext <<EOF
if ( pure_ftpd_maxcon_per_ip != "" && pure_ftpd_start_method != "st" )
then
   error "PURE_FTPD_MAXCON_PER_IP requires PURE_FTPD_START_METHOD='st'"
fi
if ( pure_ftpd_enable_virtual_users == "yes" )
then
    if ( pure_ftpd_virtual_users_n == 0 )
    then
        error "PURE_FTPD_ENABLE_VIRTUAL_USERS='yes' requires PURE_FTPD_VIRTUAL_USERS_N > 0"
    fi
fi
if ( PURE_FTPD_USER_BANDWIDTH != "" && PURE_FTPD_ANONYMOUS_BANDWIDTH != "" )
then
   error "Only PURE_FTPD_USER_BANDWIDTH or PURE_FTPD_ANONYMOUS_BANDWIDTH can be used"
fi
EOF

    # Set rights for check.exp file
    chmod 0600 /etc/check.d/${package_name}.ext
    chown root /etc/check.d/${package_name}.ext
}

# ---------------------------------------------------------------------------
# check PURE_FTPD_CERTIFICATE
# ---------------------------------------------------------------------------
check_for_link ()
{

    # check if PURE_FTPD_CERTIFICATE is a link

    cd ${cert_path}

    LINK=$(readlink $PURE_FTPD_CERTIFICATE)
    if [ "${LINK}" != '' ]; then
       PURE_FTPD_CERTIFICATE="${LINK}"
    fi

    cd - >/dev/null

}

# ---------------------------------------------------------------------------
# main
# ---------------------------------------------------------------------------
# write default config file
make_config_file /etc/default.d/${package_name}

# update from old version
read_old_variables

# write new config file
make_config_file /etc/config.d/${package_name}

# write check.d file
make_check_file

exit 0
# ---------------------------------------------------------------------------
# end
#----------------------------------------------------------------------------
