#!/bin/sh
#----------------------------------------------------------------------------
# /var/install/config.d/pure-ftpd.sh - configuration generator script for pure-ftpd
#
# Creation:     2002-11-06  jh
# Last Update:  $Id: pure-ftpd.sh 44604 2017-01-19 16:06:50Z ansgar $
#
# Copyright (c) 2002-2012 Ansgar Puester, ansgar.puester(at)freenet(dot)de
# Copyright (c) 2012-2015 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
# Copyright (c) 2016-2016 Ansgar Puester, ansgar.puester(at)freenet(dot)de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

# include eislib
. /var/install/include/eislib

package_name='pure-ftpd'

act_pmode=`get_printmode`

# set variables
basefile=/etc/config.d/base
config_file=/etc/config.d/${package_name}

xinetd_conf='/etc/xinetd.d/ftp'
services_file="/etc/services.${package_name}"

cert_path='/var/certs/ssl/certs'
certfile="$cert_path/${package_name}.pem"

# read configuration
. ${config_file}
. $basefile

# ---------------------------------------------------------------------------
# Pure-FTPd start
# ---------------------------------------------------------------------------
pure_ftpd_start ()
{
    cd /etc/rc2.d
    ln -sf ../init.d/${package_name} S50${package_name}
}

# ---------------------------------------------------------------------------
# Pure-FTPd stop
# ---------------------------------------------------------------------------
pure_ftpd_stop ()
{
    /etc/init.d/${package_name} ${__quiet} stop
    rm -f /etc/rc2.d/S??${package_name}
}

# ---------------------------------------------------------------------------
# global functions
# ---------------------------------------------------------------------------

# isnum
# =====
#
# check argument if it is numerical
#
# Call  :  isnum <argument>
#
# Return:  0  -> argument is numerical
#          1  -> argument is not numerical

isnum ()
{
  echo "$1" | grep '^[0-9][0-9]*$' > /dev/null 2>&1
  return $?
}

# ---------------------------------------------------------------------------
# handle pureftpd virtual user
# ---------------------------------------------------------------------------
handle_vftp_user ()
{
    # if something was successfully done: return 0
    # if nothing was done or error      : return 1

    vftp_group='users'
    change_list=''
    pw_change=0
    hd_change=0
    last_rc=0
    has_error=0

    if grep "^${user}:" /etc/pure-ftpd/pureftpd.passwd >> /dev/null 2>&1
    then
        # user exists

        # check what can be done
        old_home=$(grep "^${user}:" /etc/pure-ftpd/pureftpd.passwd | cut -f 6 -d ':')
        # delete trailing ./
        old_home=$(dirname "$old_home")
        new_home=$home
        if [ "$old_home" != "$new_home" ]
        then
            [ "$change_list" = '' ] && change_list='directory' ||
                change_list="$change_list directory"
            hd_change=1
        fi
        if [ "$pass" != '******' ]
        then
            [ "$change_list" = '' ] && change_list='password' ||
                change_list="$change_list password"
            pw_change=1
        fi

        if [ $hd_change = 0 -a $pw_change = 0 ]
        then
            # nothing to do
            return 1
        fi

        myecho "Modifying virtual user '$user' ($change_list)"

        # do the password change
        if [ $pw_change = 1 ]
        then
            # check for empty password
            if [ "$act_pmode" = 'html' -a "$pass" = '' ]
            then
                # webconf does not allow to ask for a password
                mecho --error "Error: virtual user $idx wrong defined"
                mecho --error "       PURE_FTPD_VIRTUAL_USERS_${idx}_USERNAME: '$user'"
                mecho --error "       PURE_FTPD_VIRTUAL_USERS_${idx}_PASSWD is empty"
                mecho --error "       password can not be changed"
                # error, modification can not be done
                return 1
            fi
            if [ -z "$pass" ]
            then
                # interactive mode
                /usr/bin/pure-pw passwd $user
                last_rc=$?
                [ $last_rc != 0 ] && has_error=1
            else
                echo -e "${pass}\n${pass}\n" |
                /usr/bin/pure-pw passwd $user > /dev/null
                last_rc=$?
                [ $last_rc != 0 ] && has_error=1
            fi
        fi
        if [ $has_error = 0 ]
        then
            # do the directory change
            if [ $hd_change = 1 ]
            then
                /usr/bin/pure-pw usermod $user -d $home
                last_rc=$?
                [ $last_rc != 0 ] && has_error=1
            fi
        fi
        if [ $has_error != 0 ]
        then
            mecho --error "Error: user '$user' was not modified (rc=$last_rc)"
            return 1
        else
            # all modification are done
            return 0
        fi

    else

        # new user
        if [ "$pass" = '******' ]
        then
            mecho --error "Error: virtual user $idx wrong defined"
            mecho --error "       PURE_FTPD_VIRTUAL_USERS_${idx}_USERNAME: '$user'"
            mecho --error "       overwritten password found, password set to ''"
            pass=''
        fi
        if [ "$act_pmode" = 'html' -a "$pass" = '' ]
        then
            # webconf does not allow to ask for a password
            mecho --error "Error: virtual user $idx wrong defined"
            mecho --error "       PURE_FTPD_VIRTUAL_USERS_${idx}_USERNAME: '$user'"
            mecho --error "       PURE_FTPD_VIRTUAL_USERS_${idx}_PASSWD is empty"
            mecho --error "       user can not be created"
            # error
            return 1
        fi

        mecho "Adding new virtual user '$user'"
        if [ "$pass" = '' ]
        then
            # ask for a password
            /usr/bin/pure-pw useradd $user -u vftp -g $vftp_group -d $home
            last_rc=$?
        else
            echo -e "${pass}\n${pass}\n" |
            /usr/bin/pure-pw useradd $user -u vftp -g $vftp_group -d $home >/dev/null
            last_rc=$?
        fi
        if [ $last_rc != 0 ]
        then
            mecho --error "Error: user was not added (rc=$last_rc)"
            return 1
        else
            return 0
        fi
    fi
}

# ---------------------------------------------------------------------------
# delete pureftpd virtual users
# ---------------------------------------------------------------------------
del_vftp_users ()
{
    del_count=0
    temp_file_config=/tmp/pureftpd.user.config.$$
    temp_file_purepw=/tmp/pureftpd.user.purepw.$$

    if [ $PURE_FTPD_ENABLE_VIRTUAL_USERS = 'yes' ]
    then
        idx=1
        myecho "Searching for deleted pureftpd's virtual users..."
        # create list from PURE_FTPD_VIRTUAL_USERS_#
        > $temp_file_config
        while [ "$idx" -le "$PURE_FTPD_VIRTUAL_USERS_N" ]
        do
            eval user='$PURE_FTPD_VIRTUAL_USERS_'$idx'_USERNAME'
            if [ "$user" != '' ]
            then
                echo "$user" >> $temp_file_config
            fi
            idx=`expr $idx + 1`
        done

        # get list from pure-pw list
        > $temp_file_purepw
        /usr/bin/pure-pw list |
        while read line
        do
            set -- $line
            user="$1"
            echo $user >> $temp_file_purepw
        done

        while read user
        do
            if ! grep "^$user\$" $temp_file_config >/dev/null 2>&1
            then
                myecho "Deleting virtual user '$user'"
                /usr/bin/pure-pw userdel $user
                handle_rc=$?
                if [ $handle_rc != 0 ]
                then
                    if [ "$act_pmode" = 'tty' ]
                    then
                        mecho --error "Error: user was not deleted (rc=$handle_rc)" < /dev/tty
                    else
                        mecho --error "Error: user was not deleted (rc=$handle_rc)"
                    fi
                else
                    del_count=`expr $del_count + 1`
                fi
            fi
        done < $temp_file_purepw

        if [ $del_count != 0 ]
        then
            myecho "Updating database /etc/pure-ftpd/pureftpd.pdb"
            /usr/bin/pure-pw mkdb
        fi

        rm -f $temp_file_config
        rm -f $temp_file_purepw

    fi
}

# ---------------------------------------------------------------------------
# create pureftpd virtual user
# ---------------------------------------------------------------------------
create_vftp_users ()
{
    # pureftpd virtual user

    mod_count=0
    myecho "Evaluating pureftpd's virtual users..."

    if ! isnum "$PURE_FTPD_VIRTUAL_USERS_N"
    then
        mecho --error "Value of PURE_FTPD_VIRTUAL_USERS_N ($PURE_FTPD_VIRTUAL_USERS_N) is undefined or not numerical"
        mecho --error "         PURE_FTPD_VIRTUAL_USERS_N was set to 0"
        PURE_FTPD_VIRTUAL_USERS_N='0'
    fi

    # vftp user and group check
    if [ $PURE_FTPD_ENABLE_VIRTUAL_USERS = 'no' \
     -a "$PURE_FTPD_VIRTUAL_USERS_N" -gt 0 ]
    then
        mecho --error "Warning: You disabled the use of virtual users having virtual users defined"
    fi

    if [ $PURE_FTPD_ENABLE_VIRTUAL_USERS = 'yes' \
     -a "$PURE_FTPD_VIRTUAL_USERS_N" = 0 ]
    then
        mecho --error "Warning: You enabled the use of virtual users having no virtual users defined"
    fi

    if [ $PURE_FTPD_ENABLE_VIRTUAL_USERS = 'yes' ]
    then
        idx=1
        while [ "$idx" -le "$PURE_FTPD_VIRTUAL_USERS_N" ]
        do
            eval active='${PURE_FTPD_VIRTUAL_USERS_'${idx}'_ACTIVE}'

            if [ "${active}" = "yes" ]
            then
                eval user='$PURE_FTPD_VIRTUAL_USERS_'$idx'_USERNAME'
                eval home='$PURE_FTPD_VIRTUAL_USERS_'$idx'_HOME'
                eval pass='$PURE_FTPD_VIRTUAL_USERS_'$idx'_PASSWD'

                if [ "$user" = '' -o "$home" = '' ]
                then
                    mecho --error "Error: virtual user $idx wrong defined"
                    mecho --error "       PURE_FTPD_VIRTUAL_USERS_${idx}_USERNAME: '$user'"
                    mecho --error "       PURE_FTPD_VIRTUAL_USERS_${idx}_HOME    : '$home'"
                else
                    handle_vftp_user && mod_count=`expr $mod_count + 1`
                fi
            fi
            idx=`expr $idx + 1`
        done
        if [ $mod_count != 0 ]
        then
            myecho "Updating database /etc/pure-ftpd/pureftpd.pdb"
            /usr/bin/pure-pw mkdb
        fi
    fi
}

# ---------------------------------------------------------------------------
# overwrite pureftpd virtual users passwords
# ---------------------------------------------------------------------------
overwrite_vftp_users_passwords ()
{
    tmp=/tmp/pure-ftpd.$$
    #tmp=inet.tmp

    # set the passwords to '******', but only, if there is one
    sed "s|^ *\(PURE_FTPD_VIRTUAL_USERS_[0-9]\+_PASSWD='\)\(.\+'\)\{1,\}\(.*$\)|\1******'\3|" ${config_file} > $tmp

    cp $tmp ${config_file}
    rm $tmp
}

# ---------------------------------------------------------------------------
# handle pureftpd virtual server
# ---------------------------------------------------------------------------
handle_virtual_server()
{
    basedir='/etc/pure-ftpd/vhosts'

    # create basedir
    if [ $PURE_FTPD_ENABLE_VIRTUAL_SERVER = 'yes' ]
    then
        myecho "Evaluating pureftpd's virtual server..."
        if [ ! -d $basedir ]
        then
            myecho --info "basedir for virtual server links '$basedir' does not exist"
            myecho --info "creating basedir $basedir ..."
            mkdir $basedir
            RC=$?
            if [ ! "$RC" = 0 ]
            then
                mecho --error "failed to create basedir $basedir (0/$RC)"
                mecho --error "PURE_FTPD_ENABLE_VIRTUAL_SERVER was set to 'no'"
                PURE_FTPD_ENABLE_VIRTUAL_SERVER='no'
            fi
        fi
    fi

    # create the links
    if [ $PURE_FTPD_ENABLE_VIRTUAL_SERVER = 'yes' ]
    then

        # fist: remove ip links in basedir
        find $basedir -type l -name '[0-9]*.[0-9]*.[0-9]*.[0-9]*' -exec rm -f {} \;

        # second: create new links
        idx=1
        while [ "$idx" -le "$PURE_FTPD_VIRTUAL_SERVER_N" ]
        do
            eval active='${PURE_FTPD_VIRTUAL_SERVER_'${idx}'_ACTIVE}'

            if [ "${active}" = "yes" ]
            then

                eval ip='$PURE_FTPD_VIRTUAL_SERVER_'$idx'_IP'
                eval dir='$PURE_FTPD_VIRTUAL_SERVER_'$idx'_DIR'
                # check directory
                if [ ! -d $dir ]
                then
                    mecho --info "directory '$dir' does not exist"
                    mecho --info "creating directory $dir ..."
                    mkdir -p $dir
                    RC=$?
                    if [ ! "$RC" = 0 ]
                    then
                        mecho --error "failed to create directory $dir  (0/$RC)"
                        mecho --error "Setting for virtual server $idx ignored"
                        dir=''
                    fi
                fi
                if [ -n "$dir" ]
                then
                    # change owner
                    chown ftp:users $dir
                    # remove probably wrong link
                    if [ -L $basedir/$ip ]
                    then
                        rm $basedir/$ip
                        rm -f /etc/pure-ftpd/${ip}
                    elif [ -f $basedir/$ip ]
                    then
                        mecho --error "$basedir/$ip exists but is not a symbolic link"
                        mecho --error "Setting for virtual server $idx ignored"
                        ip=''
                    fi
                    if [ -n "$ip" ]
                    then
                        # create link
                        if [ ! -L $basedir/$ip ]
                        then
                            ln -f -s $dir $basedir/$ip
                            rm -f /etc/pure-ftpd/${ip}
                        fi
                    fi
                fi
            fi
            idx=`expr $idx + 1`
        done
    else
        if [ -d $basedir ]
        then
            # remove ip links in basedir
            find $basedir -type l -name '[0-9]*.[0-9]*.[0-9]*.[0-9]*' -exec rm -f {} \;
            find /etc/pure-ftpd -type l -name '[0-9]*.[0-9]*.[0-9]*.[0-9]*' -exec rm -f {} \;
        fi
    fi
}


# ---------------------------------------------------------------------------
# delete xinetd ftp configuration
# ---------------------------------------------------------------------------
delete_xinetd_ftp ()
{
    if [ -f ${xinetd_conf} ]
    then
        rm -f ${xinetd_conf}
    fi

    if [ -f ${services_file} ]
    then
        rm -f ${services_file}
        /var/install/bin/update-services ${package_name}
    fi
}

# ---------------------------------------------------------------------------
# check if xinetd is enabled and running
# ---------------------------------------------------------------------------
check_xinetd ()
{
    RET=0
    if [ -f /etc/config.d/xinetd ]; then
       . /etc/config.d/xinetd
       if [ "${START_XINETD}" = 'no' ]; then
          mecho --error 'Problem: START_XINETD is set to 'no''
          RET=1
       else
          xinetd_pid=$(pgrep -f /usr/sbin/xinetd)
          if [ "${xinetd_pid}" = '' ]; then
             mecho --error 'Problem: START_XINETD is set to 'yes''
             mecho --error '         but xinetd is not running'
             RET=1
          else
             RET=0
          fi
       fi
    else
       mecho --error 'Problem: file /etc/config.d/xinetd does not exist'
       RET=1
    fi

    if [ "${RET}" = '1' ]; then
       mecho --error '         pure-ftpd will NOT be usable'
    fi

}

# ---------------------------------------------------------------------------
# create xinetd ftp configuration
# ---------------------------------------------------------------------------
create_xinetd_ftp ()
{
    if [ ${PURE_FTPD_START_METHOD} = 'xi' ]
    then
        # check if xinetd is enabled and running
        check_xinetd

        cat >${xinetd_conf} <<EOF
#
# ${xinetd_conf}
# generated by /var/install/config.d/${package_name}.sh
#
service ftp
{
  #B Expert
  #E Expert
    server = /usr/sbin/pure-ftpd
    server_args = -S ${PURE_FTPD_PORT}
    socket_type = stream
    protocol = tcp
    wait = no
    user = root
    disable = no
}
EOF

        if [ "${PURE_FTPD_PORT}" -ne "21" ]
        then

            cat >${services_file} <<EOF
#
# ${services_file}
# generated by /var/install/config.d/${package_name}.sh
#
ftp             ${PURE_FTPD_PORT}/tcp
EOF
            /var/install/bin/update-services ${package_name}
        else
            if [ -f ${services_file} ]
            then
                rm -f ${services_file}
                /var/install/bin/update-services ${package_name}
            fi
        fi
    else
        delete_xinetd_ftp
    fi
}

# ---------------------------------------------------------------------------
# check certificate file
# ---------------------------------------------------------------------------
check_certfile ()
{
    cert_name="${PURE_FTPD_CERTIFICATE}"

    if [ "$PURE_FTPD_TLS" -ne "0" ]
    then
        if [ ! -f $certfile ]
        then
            mecho --error "Error: support for SSL/TLS is enabled, but the"
            mecho --error "certificate $certfile does not exist"
            mecho --error "PURE_FTPD_TLS='${PURE_FTPD_TLS}' will be ignored."
        else
            check_pem_file $certfile
            RC=$?
            if [ ! $RC = '0' ]
            then
               mecho --error "PURE_FTPD_TLS='${PURE_FTPD_TLS}' will be ignored."
            fi
        fi
    fi
}

# ---------------------------------------------------------------------------
# handle certificate file
# ---------------------------------------------------------------------------
handle_certfile ()
{
    cert_name="${PURE_FTPD_CERTIFICATE}"
    cert_link='pure-ftpd.pem'
    cert_rehash='/usr/bin/ssl/c_rehash'

    # check, if pure-ftpd.pem has to be a link
    if [ ! "$cert_name" = "$cert_link" ]; then
        # check, if link is already correct
        cd ${cert_path}
        LINK=$(readlink $cert_link)
        # if it's not delete an recreate the link
        if [ "${LINK}" != "${cert_name}" ]; then
           rm -f ${cert_link}
           ln -sf ${cert_name} ${cert_link}
           # don't forget to rehash certificates
           ${cert_rehash} ${cert_path} >/dev/null 2>&1
        fi
        cd - >/dev/null
    fi
}

# ---------------------------------------------------------------------------
# check pem file
# ---------------------------------------------------------------------------
check_pem_file()
{
   pem_file=$1

   is_file=''
   has_hd=''

   [ -f "$pem_file" ] && is_file='Y' || is_file='N'
   grep -q '^-----BEGIN DH PARAMETERS-----' $pem_file >/dev/null 2>&1
   [ $? = 0 ] && has_hd='Y' || has_hd='N'

   my_switch="$is_file$is_link$has_hd"

   case "$my_switch" in
      YY ) # file or link and DH PARAMETERS section
           ret=0
           ;;
      YN ) # file or link but no DH PARAMETERS section
           mecho --error "Error check_pem_file: $pem_file has no DH PARAMETERS section"
           ret=1
           ;;
      *    )
           mecho --error "Error check_pem_file: Internal logic error (switch=$my_switch)"
           ret=2
           ;;

   esac
   return $ret
}


# ---------------------------------------------------------------------------
# quiet suppress all normal output
# ---------------------------------------------------------------------------
myecho ()
{
    if ! ${_quiet:-false}
    then
        mecho ${@}
    fi
}
# ---------------------------------------------------------------------------
# main
# ---------------------------------------------------------------------------

while [ ${#} -ne 0 ]
do
    case "$1" in
    --quiet)
        _quiet=true
        __quiet='--quiet'
        shift
        ;;
    *)
        _action=${1}
        shift
        ;;
    esac
done

    case "${_action}" in
    *)
        if [ "${START_PURE_FTPD}" = "yes" ]
        then
            create_vftp_users
            overwrite_vftp_users_passwords

            if [ "$PURE_FTPD_VIRTUAL_USERS_DELETE" = 'yes' ]
            then
                del_vftp_users
            fi
            handle_virtual_server

            handle_certfile
            check_certfile

            create_xinetd_ftp

            pure_ftpd_start

        else
            # first delete xinetd configuration
            # if existing
            delete_xinetd_ftp
            # then stop pure-ftpd
            pure_ftpd_stop

        fi
        ;;
    esac

exit 0
# ---------------------------------------------------------------------------
# end
# ---------------------------------------------------------------------------
